<h1>
  <br>
  <a href="#">OncoStratifier</a>
</h1> 

<h4 align="center">Oncostratifiers</h4>

<p>
    <a href="https://gitlab.fht.org/iorio-lab/OncoStratifierML/-/commits/main">
    <img src="https://img.shields.io/badge/last%20commit-september-yellow"
         alt="GitHub last commit">
    <a href="https://gitlab.fht.org/iorio-lab/OncoStratifierML/-/issues">
    <img src="https://img.shields.io/badge/open%20issues-0-green"
         alt="GitHub issues">
    <a href="https://gitlab.fht.org/iorio-lab/OncoStratifierML/-/branches">
    <img src="https://img.shields.io/badge/branches-1-blue"
         alt="GitHub branches">
    <a href="https://twitter.com/intent/tweet?text=Try this OncoStratifier Predictor AutoExec:&url=https%3A%2F%2Fgitlab.tudelft.nl%2Fytepeli%2Felsl">
    <img src="https://img.shields.io/twitter/url?url=https%3A%2F%2Fgitlab.tudelft.nl%2Fytepeli%2Felsl" alt="GitHub tweet">
    
</p>
      
<p>
  <a href="#abstract">Abstract</a> •
  <a href="#repository-description">Repository Description</a> •
  <a href="#framework-and-single-cancer-experiment">Framework and Single Cancer Experiment</a> •
  <a href="#installation-and-dependencies">Installation and Dependencies</a> •
  <a href="#installation-and-dependencies">Setting Up the Project</a> •
  <a href="#Running-OncoStratifier">Running Oncostratifier</a> •
  <a href="#direct-usage-of-OncoStratifier">Visualizing Results</a> •
  <a href="#code-integration">Code Integration</a> •
  <a href="#bibtext-reference">Bibtext Reference</a> •
  <a href="#license">License</a>
</p>


---

## Abstract

<table>
<tr>
<td>
  
Oncogenes, when mutated or overexpressed, drive tumorigenesis and lead to oncogene addiction, where cancer cells rely on these genes for survival and proliferation. Stratifying oncogene-addicted cohorts is essential for precision medicine but remains underexplored. We propose OncoStratifier, a framework to identify drugs that specifically target oncogene-addicted cancer cells, differentiating between those that induce sensitivity or resistance. Our results reveal 21,020 stratifying drugs across 267 oncogenes in 31 different cancer types. We have identified 59 mutational markers associated with 36 of these stratifying drugs across 36 distinct oncogenes and 11 cancer types. These findings underscore OncoStratifier’s potential to guide personalized cancer treatment strategies.

(Published in <a href="#">Oncostratifiers</a>)

</td>
</tr>
</table>


## Framework and Oncostratifier Result
![Framework and PIK3CA Stratifiers](framework_fig.png "Framework and PIK3CA Stratifiers")
*Oncostratifier framework and example results*
**a,** TBD **b,** TBD


## Repository Description

##### Folder hierarchy:
* **data**: Includes cached data. Raw data is in iorio/Datasets
  * **cache**: These files are not necessary, it saves some files as cache so that they can be used later on with less computational power.
* **results**: Includes results as table and also as visualization.
  * **oncostratifier_by_drug_{X}**: Found oncostratifier results when the primary group are divided into: Cell lines with {X} in genes vs cell lines without {X} in genes. {X}: 'mut' for the experiments.
    * **csv_results**: Detailed results for each oncogene addiction (Oncogene-cancer type pairs).
    * **cover**: Results of set covers.
      * **stratifying_cover_list.csv**: CSV file for list of oncogene addictions and multiple stratifying drugs found to cover oncogene addicted cohort.
      * **stratifying_cover_onlyAvailableCCLES.csv**: CSV file for list of oncogene addictions and multiple stratifying drugs found to cover oncogene addicted cohort. Only considered CCLES where there is at least 1 sensitive drug.
    * **images**: Plots from the results. Plots are pdf versioned. PNG versions are in the folder called "png" for each result.
      * **by_cancer**: The plots for circular bar graph that show Delta-Entropy for significant drugs per cancer.
      * **by_gene**: The plots for circular bar graph that show Delta-Entropy for significant drugs per gene.
      * **by_cancer_gene**: The plots for circular bar graph that show Delta-Entropy for significant drugs per cancer-gene pair.
      * **oncoscore_distribution**: Distribution of scores in each category.
      * **violin**: The violin plots that show the response of cell lines to drugs that are found significant by OncoStratifier, grouped into Oncogene-Mut and Oncogene-WT cohorts.
      * **category_distribution**: Number of stratifying drugs found for each category where the drug is targeting the specific oncogene that it is also stratifying.
      * **drug_distribution**: Histogram of drugs that are found significant in SS, SR, FS, FR.
      * **drug_cellline_heatmap**: Per cancer-oncogene, heatmaps show the binarized drug response of cell lines to each stratifiying drug.
      * **response_with_mut_marker**: Per cancer-oncogene-drug-markerGene found, scatter plots show the drug response of each cell line grouped by oncogene and marker gene. version=5 is the last version and in each subfolder, version 6 is the last version.
      * **mut_marker_dge_clustermap**: Per cancer-oncogene-marker found, clustermap shows the normalized expression values of differentially expressed genes in the OncogeneMut cohort.
      * **enrichment_analysis**: Enrichment analysis of DEGs in different gene sets.
    * **differential_analysis**: The data saved while finding mutational markers.
* **src**: Includes all the codes for the application.
  * **lib**: Includes the config and sutils for the project. It sets up important functions, locations and connection with server.
  * **process**: Includes codes to preprocess raw datasets and results.
    * **data_process**: Includes codes to preprocess raw datasets.
    * **results_process**: Includes codes to preprocess raw results from previous experiments.
  * **experiments**: Includes the codes for different experiments.
    * **oncostratifier_by_drug.py**: Finds stratifying drugs for all oncogene-cancer type pairs independently.
  * **analysis**: Includes codes to analysis and results.
    * **results_analysis**: Analyzing the results of experiments.
      * **oncostratifier_by_drug.py**: Analyzing the results of Oncostratifier.
  * **visualization**: Includes codes to visualize data and results.
    * **results_visualization**: Visualizing the results of experiments.
      * **oncostratifier_by_drug.py**: Visualizing the results of Oncostratifier.
  * **mapping**: Includes mappings between TCGA and cancer cell line dataset.


## Installation and Dependencies

##### Dependencies
* Python3.9
* requirements.txt
* or
* environemnt.yml

##### Installation
* Open terminal and go to the folder to install the project
* Clone the project: `git clone https://gitlab.fht.org/iorio-lab/OncoStratifierML.git`
* install the virtual environment library using requirements.txt: `mamba create --name oncostratifierpy39 --file requirements.txt`
* OR install the virtual environment library using yml but change the prefix in the yml file(last line) to a location you would like: `mamba env create -f environment.yml`
* activate the virtual environment: `mamba activate oncostratifierpy39`
* Check https://conda.io/projects/conda/en/latest/user-guide/tasks/manage-environments.html


## Running OncoStratifier
python3 src/experiments/oncostratifier_by_drug.py

## Visualize Results
python3 src/visualization/results_visualization/oncostratifier_by_drug.py -c [cancer] -mg [main_gene] -vt [visual_type]
* [cancer]: The cancer type, optional;
* [main_gene]: The oncogene for the first level stratification, optional; 
* [visual_type]: 'circular_bar', 'violin', 'violin_facet', 'target_match_distribution','significant_drug_distribution', 'differential_mutation','cellline_scatter_with_drug_sensitivity', 'drug_cellline_heatmap', 'response_with_mutational_signature'


## Code Integration


# Bibtex-Reference
```
@article{tepeli2021OncoStratifier,
  title={Oncostratifier},
  author={Tepeli, Yasin and Trastulla, Lucia and Goncalves, Joana},
  journal={Journal name},
  volume={00},
  number={0},
  pages={000},
  year={2024},
  publisher={publisher}
}
```

## License

[![License: LGPL v3](https://img.shields.io/badge/License-LGPL%20v3-blue.svg?style=flat-square)](https://tldrlegal.com/license/gnu-lesser-general-public-license-v3-(lgpl-3))

- Copyright © [ytepeli].
