import sys, os
sys.path.insert(0, os.getcwd())
from src.lib import config
from src.process.results_process import previous_results
from src.process.data_process import extract_features

import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt     
from sklearn.preprocessing import StandardScaler           
import umap

def find_common_genesets(gene1, gene2):
    loc = config.PERSONAL_DATA_DIR / f'raw_databases/gene_sets/c2.all.v2023.1.Hs.entrez.gmt'
    df = pd.read_csv(loc, header=None)
    df = df[0].str.split('\t', n=2, expand=True)
    geneset_dict = {}
    for idx, row in df.iterrows():
        geneset_dict[row[0]] = row[2].split('\t')
    

def cluster_categorized_genes(feature_type='gene_sequence'):
    search_folder = config.R_RESULTS_DIR
    for filepath in search_folder.rglob(f'*_entropy_2022*outputSummary.tsv'):
        df = pd.read_csv(filepath, sep='\t', index_col=None)
        cancer = filepath.stem.split('_entropy_')[1].split('_', 1)[1].rsplit('_', 1)[0]
        variant = filepath.stem.split('_entropy_')[1].split('_', 1)[0]
        main_gene = filepath.stem.split('_entropy_')[0]
        candidate_genes = df['candidateGene'].values

        genes_features_dict=None
        cache_name = f'entropy_categories_{main_gene}_{cancer}_{variant}_{feature_type}_{len(candidate_genes)}genes_prott5.npz'
        genes_features_dict = config.cache_numpy(cache_name=cache_name)
        if genes_features_dict is None:
            print(f'Sequence feature generation started! {len(candidate_genes)}')
            genes_features_dict = extract_features.call_feature_extraction(feature_type, sample_list = candidate_genes)
            config.cache_numpy(cache_name=cache_name, data=genes_features_dict)
            
        X, gene_list = genes_features_dict['features'], genes_features_dict['genes']
        print(f'Sequence feature generation ended! {len(gene_list)}')
        
        X = StandardScaler().fit_transform(X.copy())
        for min_dist in [0.0, 0.1, 0.25, 0.5, 0.8, 0.99]:
            for n_neighbors in [2, 3, 4, 5]:
                umap_params = {'min_dist': min_dist, 'n_neighbors': n_neighbors}
                umap_params_text = ''.join([f'{k[0]}{v}' for k, v in umap_params.items()])
                umap_params_text = f'UMAP_{umap_params_text}'
                pdf_loc = config.RESULTS_DIR / 'unsupervised_analysis' / 'entropy_categories' / f'{filepath.stem}_{feature_type}_{umap_params_text}_named.pdf'
                if os.path.exists(pdf_loc):
                    continue
                plt.clf()
                embedding = umap.UMAP(**umap_params).fit_transform(X.copy())
                type_categories = df.set_index('candidateGene').loc[gene_list].type.astype('category').cat
                print(type_categories.codes)
                print(df.set_index('candidateGene').loc[gene_list].type)
                #df = df[df['p']<0.05]
                fig, ax = plt.subplots()
                scatter = plt.scatter(
                    embedding[:, 0], embedding[:, 1], c=type_categories.codes
                )
                
                for i in range(len(gene_list)): 
                    plt.annotate(gene_list[i], (embedding[i, 0], embedding[i, 1] + 0.2)) 
                title = cancer + ', ' + main_gene + ', ' + variant
                plt.title(title)
                plt.xlabel('UMAP 1')
                plt.ylabel('UMAP 2')
                sns.despine()
                handles, labels = scatter.legend_elements(prop="colors", alpha=0.6)
                legend2 = ax.legend(handles, type_categories.categories, loc="upper right", title="Category")
                config.ensure_dir(pdf_loc)
                plt.savefig(pdf_loc, dpi=300, bbox_inches='tight')

def main():
    cluster_categorized_genes()
    
if __name__ == "__main__":
    main()