import os

from src.lib.sutils import *
import lzma
import pickle

ROOT_DIR = os.environ.get('ONCOSTRATIFIER_ROOT_DIR', None)
if ROOT_DIR is not None:
    ROOT_DIR = Path(ROOT_DIR)

if ROOT_DIR is None or not ROOT_DIR.exists() or not ROOT_DIR.is_dir():
    ROOT_DIR = Path(__file__).resolve().parent.parent.parent

if os.path.exists('/group/iorio/'):
    GROUP_DIR = str2path('/group/iorio')
elif os.path.exists('/Volumes/iorio/'):
    GROUP_DIR = str2path('/Volumes/iorio')
    

FHT_ROOT_DIR = GROUP_DIR / 'yasin.tepeli' / 'python_projects' / 'OncoStratifierML'
ROOT_DIR = FHT_ROOT_DIR
DATA_DIR = ROOT_DIR / 'data'
CACHE_DIR = DATA_DIR / 'cache'
RESULTS_DIR = ROOT_DIR / 'results'
PLOTS_DIR = ROOT_DIR / 'plots'

FHT_ROOT_DIR = GROUP_DIR / 'yasin.tepeli' / 'python_projects' / 'OncoStratifierML'
FHT_DATA_DIR = FHT_ROOT_DIR / 'data'
FHT_RESULTS_DIR = FHT_ROOT_DIR / 'results'
FHT_PLOTS_DIR = FHT_ROOT_DIR / 'plots'

R_DIR = GROUP_DIR / 'lucia' / 'oncostratifier'
R2_DIR = GROUP_DIR / 'yasin.tepeli' / 'r_projects' / 'oncostratifier'
RAW_DATA_DIR = GROUP_DIR / 'Datasets'
PERSONAL_DATA_DIR = GROUP_DIR / 'yasin.tepeli' / 'datasets'
R_RESULTS_DIR = R_DIR / 'results'
R_SCREEN_DIR = R_DIR / 'ScreenGene'
R2_RESULTS_DIR = R2_DIR / 'results'
R2_SCREEN_DIR = R2_DIR / 'ScreenGene'
safe_create_dir(DATA_DIR)
safe_create_dir(RESULTS_DIR)
safe_create_dir(PLOTS_DIR)


def get_safe_data_file(file_path: Path):
    """
    Parameters
    ----------
    file_path: :obj:`pathlib.Path`
    Data file path, if it is absolute will use as is, otherwise will look under data folder
    Returns
    -------
    normalized data file path
    """
    file_path = get_safe_path_obj(file_path)
    if file_path.is_absolute():
        return file_path
    return DATA_DIR / file_path

def ensure_dir(file_path):
    directory = os.path.dirname(file_path)
    if not os.path.exists(directory):
        os.makedirs(directory, exist_ok=True)

def cache_numpy(cache_name, data=None, is_pickle=False, is_full_loc=False):
    if is_full_loc:
        cache_loc = cache_name
    else:
        cache_loc = DATA_DIR / 'cache' / cache_name
    ensure_dir(cache_loc)
    if data is not None:
        if is_pickle:
            with lzma.open(cache_loc, "wb") as f:
                pickle.dump(data, f)
        else:
            np.savez_compressed(cache_loc, **data)
    elif os.path.exists(cache_loc):
        if is_pickle:
            with lzma.open(cache_loc, "rb") as f:
                return pickle.load(f)
        else:
            return np.load(cache_loc, allow_pickle=True)
    else:
        print(f'No cache ({cache_loc}) found!')