import pandas as pd
from src.lib import config

def map_tcga_oncotreeprimary(df):
    keyword2tcga = {'breast': 'BRCA', 'cervi': 'CESC', 'ovar': 'OV', 'colo': 'COAD',
                    'glioma': 'GBM', 'renal': 'KIRC', 'leukemia': 'LAML', 'myeloproliferative': 'LAML',
                    'pancrea': 'PAAD', 'lung': 'LUAD', 'prostate': 'PRAD', 'melanoma': 'SKCM', 'merkel': 'SKCM',
                    'cutaneous': 'SKCM', 'ocular': 'UVM', 'retino': 'UVM'}
    ccle2tcga = {}
    tcga2ccle = {}
    for idx, row in df.iterrows():
        ccle = row['ModelID']
        matches = [t for k, t in keyword2tcga.items() if k in str(row['OncotreePrimaryDisease']).lower()]
        if len(matches)==1:
            tcgasymbol = matches[0]
        elif len(matches)>1 and 'ocular' in str(row['OncotreePrimaryDisease']).lower():
            tcgasymbol = 'UVM'
        if len(matches)!=1:
            ccle2tcga[ccle] = tcgasymbol
            if tcgasymbol not in tcga2ccle.keys():
                tcga2ccle[tcgasymbol] = [ccle]
            else:
                tcga2ccle[tcgasymbol].append(ccle)

    return ccle2tcga, tcga2ccle

def depmap2any(aim='CellLineName', reverse=True, upper=True):
    model_loc = config.DATA_DIR / 'depmap_23Q2' / 'Model.csv'
    df = pd.read_csv(model_loc)
    if upper:
        df[aim] = df[aim].str.upper()
    if not reverse:
        return df.set_index('ModelID')[aim].to_dict()
    else:
        return df.set_index(aim)['ModelID'].to_dict()
    

def main():
    name2modelid = depmap2any()
    
if __name__ == "__main__":
    main()
    
'''
array(['CL-14', 'CL-34', 'CL-40', 'COLO-320-HSR', 'CW-2', 'Geo',
       'Hs-255-T', 'Hs-675-T', 'LS-123', 'NCI-H508', 'NCI-H684',
       'SK-CO-1', 'SNU-1040', 'SNU-1197', 'SNU-175', 'SNU-283', 'SNU-407',
       'SNU-C2A', 'SNU-C2B', 'SW1116', 'SW1417', 'SW948'], dtype='<U12')
'''