import pandas as pd
import numpy as np
from sklearn.linear_model import LogisticRegression, LogisticRegressionCV, ElasticNetCV, ElasticNet
from sklearn.model_selection import StratifiedKFold, KFold
R_STATE=123

def call_LR(version=None, random_state=R_STATE, **params):
    if version is None:
        return LogisticRegression(**params)
    elif version=='0':
        LogisticRegression(penalty='elasticnet', C=0.5, class_weight='balanced', n_jobs=-1, random_state=random_state,
                            solver='saga', max_iter=100, l1_ratio=0.5)
    elif version=='1':
        LogisticRegression()


def call_LRCV(version=None, random_state=R_STATE, **params):
    if version is None:
        return LogisticRegressionCV(**params)
    elif version=='0':
        return LogisticRegressionCV(penalty='elasticnet', Cs=10, class_weight='balanced', n_jobs=-1, random_state=random_state,
                            solver='saga', max_iter=100, l1_ratios=[.1, .5, .7, .9, .95, .99, 1],
                            cv=StratifiedKFold(n_splits=5, random_state=random_state, shuffle=True))
    elif version=='1':
        LogisticRegression()
        

def call_ENCV(version=None, random_state=R_STATE, **params):
    if version is None:
        return ElasticNetCV(**params)
    elif version=='0':
        return ElasticNetCV(l1_ratio=[.1, .5, .7, .9, .95, .99, 1], n_alphas=100, n_jobs=-1, random_state=random_state,
                            cv=StratifiedKFold(n_splits=5, random_state=random_state, shuffle=True))
    elif version=='1':
        return ElasticNetCV(l1_ratio=[.1, .5, .7, .9, .95, .99, 1], n_alphas=100, n_jobs=-1, random_state=random_state,
                            cv=KFold(n_splits=5, random_state=random_state, shuffle=True))
            

def call_model(model_name, **params):
    if 'random_state' in params:
        random_state=params['random_state']
    else:
        random_state=R_STATE
    
    if '_v':
        version=model_name.split('_v')[1]
    else:
        version=None
    
    if 'LRCV' in model_name:
        return call_LRCV(version, random_state, **params)
    elif 'LR' in model_name:
        return call_LR(version, random_state, **params)
    elif 'ENCV' in model_name:
        return call_ENCV(version, random_state, **params)
        