import rpy2
from rpy2 import robjects as ro
import rpy2.robjects.packages as rop
from rpy2.robjects.vectors import StrVector

class MyREnvironment(object):
    def __init__(self):
        self.import_packages(base=True)
        rprint = ro.globalenv.find("print")
        
    def import_packages(self, packages = [], base=False):
        if base:
            # import R's "base" and "utils" package
            self.base = rop.importr('base')
            self.utils = rop.importr('utils')
            self.utils.chooseCRANmirror(ind=1) #Select the first mirror

        self.install_packages(packages)
        for package_name in packages:
            rop.importr(package_name)
        
    def install_packages(self, package_names=[]):
        # Selectively install what needs to be install.
        names_to_install = [x for x in package_names if not rop.isinstalled(x)]
        if len(names_to_install) > 0:
            self.utils.install_packages(StrVector(names_to_install))

    def get_object(self, obj_name):
        if len(ro.r[obj_name])==1:
            return ro.r[obj_name][0]
        return ro.r[obj_name]


    def create_code_snippet(self, the_code=None):
        my_r_func = ro.r( '''
                        # create a function `f`
                        f <- function(r, verbose=FALSE) {
                            if (verbose) {
                                cat("I am calling f().\n")
                            }
                            2 * pi * r
                        }
                        # call the function `f` with argument value 3
                        f(3)
                    ''')
        return my_r_func#ro.globalenv[func_name]

    def get_function(self, func_name):
        return ro.r[func_name]#ro.globalenv[func_name]

def main():
    r_env = MyREnvironment()
    r_env.install_packages(['edgeR'])
    print()
    
if __name__ == "__main__":
    main()