import os
project_path = '.'
path2this = os.path.dirname(os.path.abspath(__file__)).split('/')
for i, folder in enumerate(path2this):
    if folder.lower() == 'oncostratifierml':
        project_path = '/'.join(path2this[:i + 1])

import numpy as np
import pandas as pd
from src.lib import config
from src.mapping import ccle_mappings
from src.process.results_process import previous_results
import pyreadr

BEM_FOLDER = config.GROUP_DIR / 'lucia' / 'datasets' / 'BEM_Iorio_2016'
tcga2ccle =  {'ACC':'Other Solid Cancers', 'ALL': 'T-Lymphoblastic Leukemia', 'BLCA':'Bladder Carcinoma', 'BRCA': 'Breast Carcinoma', 
  'CESC':'Cervical Carcinoma', 'COREAD': 'Colorectal Carcinoma', 'DLBC':"B-Cell Non-Hodgkin's Lymphoma", 
  'ESCA':'Esophageal Squamous Cell Carcinoma', 'GBM':'Glioblastoma', 'HNSC':'Head and Neck Carcinoma', 
  'KIRC':'Kidney Carcinoma', 'LAML':'Acute Myeloid Leukemia', 'LCML':'Chronic Myelogenous Leukemia', 
  'LGG': 'Glioblastoma', 'LIHC':'Hepatocellular Carcinoma', 'LUAD':'Non-Small Cell Lung Carcinoma', 
  'LUSC':'Squamous Cell Lung Carcinoma', 'MB':'Other Solid Cancers', 'MESO':'Mesothelioma', 'MM':'Plasma Cell Myeloma', 
  'NB':'Neuroblastoma', 'OV':'Ovarian Carcinoma', 'PAAD':'Pancreatic Carcinoma', 'PRAD':'Prostate Carcinoma', 
  'SCLC':'Small Cell Lung Carcinoma', 'SKCM':'Melanoma', 'STAD':'Gastric Carcinoma', 
  'THCA':'Thyroid Gland Carcinoma', 'UCEC':'Endometrial Carcinoma'}
ccle2tcga =  {item:key for key, item in tcga2ccle.items()}

def get_cfe(source='cellline', data_type='CG', cancer='PANCAN'):
    """
    The function `get_cfe` returns a pandas DataFrame by reading a file based on the provided
    parameters.
    
    :param source: The source parameter specifies the source of the data. It can be either 'cellline'
    for cell line data or 'primary' or 'tumour' for primary tumor data, defaults to cellline (optional)
    :param data_type: The `data_type` parameter specifies the type of data to retrieve. It can have two
    possible values: 'CG' or 'CNV', defaults to CG (optional)
    :param cancer: The `cancer` parameter is used to specify the type of cancer. It is a string that
    represents the cancer type. In the code, it is used to construct the file name for the BEM file that
    will be read, defaults to PANCAN (optional)
    :return: a pandas DataFrame that is read from a file specified by the variable `cfe_loc`.
    """
    if 'cellline' in source.lower():
        if data_type=='CG':
            cfe_loc = BEM_FOLDER / 'CellLines_CG_BEMs' / f'{ccle2tcga[cancer]}_SEQ_BEM.txt'
        elif data_type=='CNV':
            cfe_loc = BEM_FOLDER / 'CellLine_CNV_BEMs' / f'{cancer}_CNA_BEM.rdata.txt'
    elif 'primary' in source.lower() or 'tumour' in source.lower():
        if data_type=='CG':
            cfe_loc = BEM_FOLDER / 'PrimTum_CG_BEMs_cf' / f'_cf_{cancer}.txt'
        elif data_type=='CNV':
            cfe_loc = BEM_FOLDER / 'Tumours_CNV_BEMs' / f'{cancer}_CNA_BEM.rdata.txt'
    
    return pd.read_csv(cfe_loc, sep='\t')
    
def main():
    df = get_cfe('cellline', 'CG', 'Colorectal Carcinoma')
    print()
    
if __name__ == "__main__":
    main()