import sys, os
sys.path.insert(0, os.getcwd())
sys.path.append('/Users/yitepeli/.local/lib/python3.9/site-packages')
import pandas as pd
import numpy as np
from src.lib import config
from pathlib import Path

MSIGDB_DATA_DIR = config.RAW_DATA_DIR / 'MSigDB'
GENESET_DATA_DIR = MSIGDB_DATA_DIR / 'msigdb_v2023.2.Hs_GMTs'

def get_geneset(name='all', group='canonical_pathway', return_links=False):
    in_loc = get_locs()[group][name]
    geneset2gene = {}
    geneset2link = {}
    with open(in_loc, 'r') as f:
        all_genesets = f.readlines()
    for geneset in all_genesets:
        geneset_splitted = geneset.replace('\n', '').split('\t')
        geneset_name, geneset_link, geneset_genes = geneset_splitted[0], geneset_splitted[1], geneset_splitted[2:]
        geneset2gene[geneset_name] = geneset_genes
        geneset2link[geneset_name] = geneset_link
    if return_links:
        return geneset2gene, geneset2link
    else:
        return geneset2gene
    
def generate_gene2geneset(geneset2gene):
    unique_genes = np.unique(np.concatenate(list(geneset2gene.values()), axis=None))
    gene2geneset = {}
    for agene in unique_genes:
        gene2geneset[agene] = [geneset_name for geneset_name, geneset_genes in geneset2gene.items() if agene in geneset_genes]
    return gene2geneset

def get_locs():
    loc_dict = {}
    loc_dict['canonical_pathway']={}
    for identifier in ['entrez', 'symbols']:
        loc_dict['canonical_pathway']['all'] = GENESET_DATA_DIR / f'c2.cp.v2023.2.Hs.{identifier}.gmt'
        for pw_name in ['kegg_legacy', 'kegg_medicus', 'biocarta', 'pid', 'reactome', 'wikipathways']:
            loc_dict['canonical_pathway'][pw_name] = GENESET_DATA_DIR / f'c2.cp.{pw_name}.v2023.2.Hs.{identifier}.gmt'

    return loc_dict


def get_all_pathway_dicts():
    geneset_name, geneset_group = 'all', 'canonical_pathway'
    geneset_dicts_loc = f'MSigDB/genesetdicts_{geneset_group}_{geneset_name}.pkl'
    geneset_conversions = config.cache_numpy(geneset_dicts_loc, is_pickle=True, is_full_loc=False)
    if geneset_conversions is None:
        geneset2gene = get_geneset(name=geneset_name, group=geneset_group, return_links=False)   
        gene2geneset = generate_gene2geneset(geneset2gene)
        geneset_conversions = {'geneset2gene':geneset2gene, 'gene2geneset': gene2geneset}
        config.cache_numpy(geneset_dicts_loc, data=geneset_conversions, is_pickle=True, is_full_loc=False) 
    geneset2gene, gene2geneset = geneset_conversions['geneset2gene'], geneset_conversions['gene2geneset']
    return geneset2gene, gene2geneset


def main():
    colors = ['#7570b3', '#d95f02']
    colors2= ['#DDAA33', '#117733']
    group_labels =[f'WT', f'Mut']
    cancer_oncogene_marker_dict = {
                                   'Acute Myeloid Leukemia':('laml',{'FLT3': ['NRAS'], 'MGAM': ['TP53']}), 
                                   "B-Cell Non-Hodgkin's Lymphoma":('dlbc',{'REV3L': ['ARID1A'], 'VAV1': ['TP53'], 'ALK': ['TP53']}), 
                                   'Breast Carcinoma':('brca',{'BRD4': ['PIK3R1'], 'ROS1': ['CHD9'], 'MSI2': ['NR4A2']}),
                                   'Colorectal Carcinoma':('coadread',{'KRAS': ['PPP2R1A', 'ITSN1', 'KMT2D', 'NF1', 'AKAP9'], 'JAK3': ['MAP2K1'],
                                                                       'UBR5': ['FXR1'], 'MYBL1': ['MAP2K1'], 'KSR2': ['FXR1'], 'GNAS': ['FXR1'],
                                                                       'WWP1': ['FXR1', 'PIK3CA'], 'MECOM': ['FXR1'], 'NUP98': ['CNOT1', 'PIK3CA', 'PTEN'],
                                                                       'NTRK3': ['FXR1'], 'MTOR': ['FXR1']}), 
                                   'Esophageal Squamous Cell Carcinoma':('esca',{'WWTR1': ['EP300']}), 
                                   'Gastric Carcinoma':('stad',{'NSD2': ['ARID1A'], 'MECOM': ['EIF4G1']}), 
                                   'Glioblastoma':('gbm',{'IGF1R': ['EGFR']}), 
                                   'Melanoma':('skcm',{'MSI2': ['SVEP1'], 'PAK5': ['BPTF'], 'JAK3': ['ANK3']}), 
                                   'Non-Small Cell Lung Carcinoma':('luad',{'ADHFE1': ['STK11'], 'PDGFRB': ['MAP4K3'], 'NTRK3': ['KEAP1'], 
                                                                             'LGR5': ['CLSPN'], 'KDM5A': ['ARID1A'], 'NRAS': ['SETD2'],
                                                                             'MAP3K13': ['CLSPN'], 'MECOM': ['TP53BP1'], 'MGAM': ['TP53BP1']}), 
                                   'Ovarian Carcinoma':('ov',{'BCR': ['BRCA2', 'PIK3CA', 'SETD2'], 'ROS1': ['ATRX'],
                                                              'BRAF': ['PIK3CA', 'KMT2D'], 'LRP6': ['PIK3CA'], 'RPTOR': ['PIK3CA'], 'JAK3': ['PIK3CA'], 'VAV2': ['PIK3CA']}),  
                                   'Small Cell Lung Carcinoma':('luad',{'PAK5': ['BAZ2B', 'KMT2A'], 'CARD11': ['TJP1'], 'INSR': ['MED12']})
                                   }
    res_dict = {}
    for cancer, (cancer_tcga, oncogene_marker_dict) in cancer_oncogene_marker_dict.items():
        for oncogene, markers in oncogene_marker_dict.items():
            for marker in markers:
                    res_per_dge_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_mut' / 'images' / 'mut_marker_dge_clustermap' / 'data' / f'{cancer}_{oncogene}_{marker}_deseq2_noxticks.pkl'
                    dict_per_dge_res = config.cache_numpy(res_per_dge_loc, is_pickle=True, is_full_loc=True)
                    significant_dge_df = dict_per_dge_res['dge_df'] 
                    exp_scaled_counts_df = dict_per_dge_res['expression_zscored'] 
                    metadata_df = dict_per_dge_res['metadata'] 
                    
                    col_lut = dict(zip(metadata_df[f'{marker} Status'].unique(), colors))
                    col_colors = metadata_df[f'{marker} Status'].map(col_lut)
                    
                    row_lut = dict(zip(significant_dge_df[f'LFC'].unique(), colors2))
                    row_colors = significant_dge_df[f'LFC'].map(row_lut)
                    
                    g = sns.clustermap(exp_scaled_counts_df, col_colors=col_colors, cmap="coolwarm", center=0, col_cluster=False, row_colors=row_colors)

    print()


if __name__ == '__main__':
    main()
