import os
project_path = '.'
path2this = os.path.dirname(os.path.abspath(__file__)).split('/')
for i, folder in enumerate(path2this):
    if folder.lower() == 'oncostratifierml':
        project_path = '/'.join(path2this[:i + 1])

import numpy as np
import pandas as pd
from src.lib import config

FPMTB_FOLDER = config.RAW_DATA_DIR / 'FPMTB'

def get_drug_response(threshold=None, remove_healthy=True):
    """_summary_
    Args:
        source (str, optional): Models or Samples. Defaults to 'model'.
        score (str, optional): AUC or iC50. Defaults to 'iC50'.
        return_matrix (boolean, optional): Converting the matrix to drugXmodel matrix with "score" values. Defaults to 'True'.

    Returns:
        dataframe: Loads the dataframe
    """
    in_loc = FPMTB_FOLDER / f'File_3_Drug_response_sDSS_164S_17Healthy.xlsx'
    df = pd.read_excel(in_loc)
    df = df.set_index('Drug_name').drop(columns=['Drug_ID']).astype(float)
    if remove_healthy:
        df = df[[col for col in df.columns if 'Healthy' not in col]]
    if threshold is not None:
        binary_df = (df >= threshold).astype(int)
        binary_df[df.isna()] = np.NaN
        return binary_df
    return df

def get_mutation(source='VAF'):
    """_summary_
    Args:
        source (str, optional): Models or Samples. Defaults to 'model'.
        score (str, optional): AUC or iC50. Defaults to 'iC50'.
        return_matrix (boolean, optional): Converting the matrix to drugXmodel matrix with "score" values. Defaults to 'True'.

    Returns:
        dataframe: Loads the dataframe
    """
    if source == 'VAF':
        in_loc = FPMTB_FOLDER / f'File_5_Mutation_Variant_Allele_Frequency_225S_340G.csv' 
        df = pd.read_csv(in_loc, sep=',', index_col=0).notnull().astype("int")
    elif source == 'Binary':
        in_loc = FPMTB_FOLDER / f'File_6_Binary_mutation_225S_57G.xlsx'
        df = pd.read_excel(in_loc)
    return df
    
def main():
    """
    The main function performs an investigation by matching genes and identifying top drugs based on
    drug response data.
    """
    threshold = 8.7
    df_response_sdss = get_drug_response(threshold=threshold)
    df_mut = get_mutation(source='VAF')
    investigation_list = [('MGAM', '1047_Nutlin-3a (-)_GDSC2', 'TP53'), ('FLT3', '63_BMS-509744_GDSC1', 'NRAS')]
    for screen_gene, stratifying_drug, marker_gene in investigation_list:
        match_list = []
        for firstgene in [agene1 for agene1 in df_mut.index[df_mut.index.notnull()] if screen_gene in agene1]:
            for secondgene in [agene2 for agene2 in df_mut.index[df_mut.index.notnull()] if marker_gene in agene2]:
                match_cols = df_mut.columns[(df_mut.loc[firstgene] & df_mut.loc[secondgene])==1].values
                if len(match_cols)>0:
                    print(f'{firstgene}, {secondgene}: {match_cols}')
                    match_list += list(match_cols)
        unique_pdxs = np.unique(match_list)
        print(unique_pdxs)
        for aimpdx in unique_pdxs:
            #aimpdx = aimpdx.split('-')[0]
            match_pdxs = [apdx for apdx in df_response_sdss.columns if aimpdx in apdx]
            for match_pdx in match_pdxs:
                top5drugs_auc = df_response_sdss[match_pdx].sort_values(ascending=False).iloc[:5].index
                print(f'AUC-{aimpdx}-{match_pdx}\n{df_response_sdss.loc[top5drugs_auc, match_pdx]}')
                print('\n------------------------------------------------------\n')
        
    
if __name__ == "__main__":
    main()