import sys, os
sys.path.insert(0, os.getcwd())
sys.path.append('/Users/yitepeli/.local/lib/python3.9/site-packages')
import pandas as pd
import numpy as np
import csv
import collections as coll
from src.lib import config
from pathlib import Path
import pickle as pkl
import warnings
import requests
import seaborn as sns
import matplotlib.pyplot as plt     
from matplotlib.patches import Patch
from sklearn.preprocessing import StandardScaler           
import umap

from pydeseq2.dds import DeseqDataSet
from pydeseq2.default_inference import DefaultInference
from pydeseq2.ds import DeseqStats
from pydeseq2.utils import load_example_data

warnings.filterwarnings("ignore", category=UserWarning)
warnings.filterwarnings("ignore", category=RuntimeWarning)
warnings.filterwarnings("ignore", category=DeprecationWarning)
warnings.filterwarnings("ignore", category=FutureWarning)

from matplotlib import rcParams
rcParams['pdf.fonttype'] = 42
rcParams['ps.fonttype'] = 42
rcParams['font.sans-serif'] = "Arial"
rcParams['font.family'] = "Arial"

CBIOPORTAL_DATA_DIR = config.RAW_DATA_DIR / 'cBioPortalData'
config.ensure_dir(CBIOPORTAL_DATA_DIR)
#CANCERS = ['BRCA', 'CESC', 'COAD', 'GBM', 'KIRC', 'LAML', 'LUAD', 'OV', 'PAAD', 'PRAD', 'SKCM', 'UVM']
CANCERS = ['BRCA', 'CESC', 'COAD', 'KIRC', 'LAML', 'LUAD', 'OV', 'SKCM']
#CANCERS = ['KIRC', 'LAML', 'LUAD', 'OV']

def connect2cbioportal_api(keywords=None, verbose=True, download_all=False):
    from bravado.client import SwaggerClient
    cbioportal = SwaggerClient.from_url('https://www.cbioportal.org/api/v2/api-docs',
                                    config={"validate_requests":False,"validate_responses":False,"validate_swagger_spec":False})
    if verbose:
        studies = cbioportal.Studies.getAllStudiesUsingGET().result()
        cancer_types = cbioportal.Cancer_Types.getAllCancerTypesUsingGET().result()

        print("In total there are {} studies in cBioPortal, spanning {} different types of cancer.".format(
            len(studies),
            len(cancer_types)
        ))
        for study in studies: print(f"The total number of samples in {study.name} ({study.studyId}) is: {study.allSampleCount}") 
        for study in studies: print(f"TCGA study {study.name} is: {study.studyId}") if ('TCGA' in study.description and 'Firehose' in study.description) else None
    studies=  []
    if keywords is not None:
        genes = cbioportal.Genes.getAllGenesUsingGET().result()
        all_gene_hg_symbols = [gene.hugoGeneSymbol for gene in genes]
        all_gene_entrez = [gene.entrezGeneId for gene in genes]
        for study in cbioportal.Studies.getAllStudiesUsingGET().result(): #print(f"TCGA study {study.name} is: {study.studyId}") if ('TCGA' in study.description and 'Firehose' in study.description) else None
            if download_all:
                url_address = f'https://cbioportal-datahub.s3.amazonaws.com/{study.studyId}.tar.gz'
                save_loc = TCGA_FIREHOSE_DATA_DIR / f'{study.studyId}'
                if not save_loc.exists():
                    response = requests.get(url_address, stream=True)
                    if response.status_code == 200:
                        with open(save_loc, 'wb') as f:
                            f.write(response.raw.read())
            if all([keyword.lower() in study.description.lower() for keyword in keywords]):
                print({study.studyId})
                studies.append(study.studyId)
                '''
                molecular_profiles = cbioportal.Molecular_Profiles.getAllMolecularProfilesInStudyUsingGET(studyId = study.studyId).result()
                for molecular_profile in molecular_profiles:
                    samples = cbioportal.Sample_Lists.getAllSampleIdsInSampleListUsingGET(sampleListId=molecular_profile.molecularProfileId).result()
                    data = cbioportal.Molecular_Data.fetchAllMolecularDataInMolecularProfileUsingPOST(
                        molecularProfileId = molecular_profile.molecularProfileId, 
                        molecularDataFilter={'entrezGeneIds': all_gene_entrez, 'sampleIds':samples}).result()
                    print()
                '''
    print(studies)
    return cbioportal

def load_mut(cancer, symbol2entrez=None, remove_silent=True):
    file_loc = CBIOPORTAL_DATA_DIR / f'{cancer.lower()}_tcga_pan_can_atlas_2018' / 'data_mutations.txt'
    df = pd.read_csv(file_loc, sep='\t')
    df = df[['Hugo_Symbol', 'Entrez_Gene_Id', 'Variant_Classification', 'Variant_Type', 'Tumor_Sample_Barcode', 'Matched_Norm_Sample_Barcode']]
    if remove_silent:
        df = df[df['Variant_Classification'] != 'Silent']
    if symbol2entrez is not None:
        df['Entrez_Gene_Id'] = df.set_index('Hugo_Symbol')['Entrez_Gene_Id'].fillna(symbol2entrez).values
    return df


def load_expr_cnv(file_loc, symbol2entrez=None, is_entrez = True):
    df = pd.read_csv(file_loc, sep='\t')
    df['Hugo_Symbol'] = df['Hugo_Symbol'].str.split('|', n=1, expand=True)[0]
    if is_entrez:
        if symbol2entrez is not None:
            df['Entrez_Gene_Id'] = df.set_index('Hugo_Symbol')['Entrez_Gene_Id'].fillna(symbol2entrez).values
        df['Entrez_Gene_Id'] = df['Entrez_Gene_Id'].fillna(0).astype(int).astype(str)
        df = df[df['Entrez_Gene_Id']!='0']
        #df = df['Entrez_Gene_Id'].fillna(0).astype(int).astype(str)
        df = df.set_index('Entrez_Gene_Id')
        df = df.drop(columns='Hugo_Symbol')
    else:
        df = df.set_index('Hugo_Symbol')
        df = df.drop(columns='Entrez_Gene_Id')
    #df = df.drop(columns='Entrez_Gene_Id')
    #df_upd = df.rename(columns=mapping)
    #df_upd = df_upd.sort_index()
    #df_upd = df_upd.sort_index(axis=1)
    df = df.sort_index()
    df = df.sort_index(axis=1)
    return df


def load_expr(cancer, is_entrez = False, z_scored = False):
    if z_scored:
        file_loc = CBIOPORTAL_DATA_DIR / f'{cancer.lower()}_tcga_pan_can_atlas_2018' / 'data_mrna_seq_v2_rsem_zscores_ref_all_samples.txt' 
    else:
        file_loc = CBIOPORTAL_DATA_DIR / f'{cancer.lower()}_tcga_pan_can_atlas_2018' / 'data_mrna_seq_v2_rsem.txt'
    df = pd.read_csv(file_loc, sep='\t')
    if 'Hybridization REF' in df.columns:
        df[['Hugo_Symbol', 'Entrez_Gene_Id']] = df['Hybridization REF'].str.split('|', n=1, expand=True)
        df = df.drop(index=0).drop(columns=['Hybridization REF'])
    if is_entrez:
        df = df.set_index('Entrez_Gene_Id')
        df = df.drop(columns='Hugo_Symbol')
    else:
        df = df.set_index('Hugo_Symbol')
        df = df.drop(columns='Entrez_Gene_Id')
    df = df.astype(float)
    #df_upd = df.rename(columns=mapping)
    #df_upd = df_upd.sort_index()
    #df_upd = df_upd.sort_index(axis=1)
    df = df.sort_index()
    df = df.sort_index(axis=1)
    return df


def load_patient_info(cancer):
    file_loc = CBIOPORTAL_DATA_DIR / f'{cancer.lower()}_tcga_pan_can_atlas_2018' / 'data_clinical_patient.txt'
    df = pd.read_csv(file_loc, sep='\t', header=4)
    df = df[['PATIENT_ID', 'SEX', 'RACE', 'ETHNICITY', 'AGE', 'OS_STATUS', 'OS_MONTHS']]
    df['OS_STATUS'] = df['OS_STATUS'].str.split(':', n=1, expand=True)[0]
    return df


def load_sample_info(cancer, only_primary=True):
    file_loc = CBIOPORTAL_DATA_DIR / f'{cancer.lower()}_tcga_pan_can_atlas_2018' / 'data_clinical_sample.txt'
    df = pd.read_csv(file_loc, sep='\t', header=4)
    if only_primary: 
        df = df[df['SAMPLE_ID'].str[-2:]=='01']
    df = df[['PATIENT_ID', 'SEX', 'RACE', 'ETHNICITY', 'AGE', 'OS_STATUS', 'OS_MONTHS']]
    df['OS_STATUS'] = df['OS_STATUS'].str.split(':', n=1, expand=True)[0]
    return df


def load_gtex_expr(file_loc):
    expr = pd.read_csv(file_loc, sep='\t', header=2)
    return expr


def load_cancer_gtex_expr(file_loc):
    df = pd.read_csv(file_loc, sep='\t')
    if 'Name' in df.columns:
        df = df.drop(columns=['Name'])
    if 'Unnamed: 0' in df.columns:
        df = df.drop(columns=['Unnamed: 0'])
    df = df.set_index('Description')
    df = df.astype(float)
    df = df.sort_index()
    df = df.sort_index(axis=1)

    return df


def load_gtex_sample_info(file_loc):
    gtex_samples = pd.read_csv(file_loc, sep='\t')
    gtex_short = gtex_samples[['SAMPID', 'SMTSD']]
    cancer2tissue = {'BRCA': 'Breast - Mammary Tissue', 'CESC': ['Cervix - Ectocervix', 'Cervix - Endocervix'],
                     'COAD': 'Colon - Transverse', 'KIRC': 'Kidney - Cortex', 'LAML': 'Whole Blood', 'LUAD': 'Lung',
                     'OV': 'Ovary', 'SKCM': ['Skin - Not Sun Exposed (Suprapubic)', 'Skin - Sun Exposed (Lower leg)']}
    cancer_pat_dict = coll.OrderedDict()
    for cancer, tissue in cancer2tissue.items():
        if type(tissue) == str:
            cancer_pat_dict[cancer] = gtex_short[gtex_short['SMTSD'] == tissue]['SAMPID'].values
        else:
            cancer_pat_dict[cancer] = gtex_short[(gtex_short['SMTSD'] == tissue[0]) | (gtex_short['SMTSD'] == tissue[1])]['SAMPID'].values

    return cancer_pat_dict


def convert_mut2bin(mutdf, genecol='Entrez_Gene_Id', patcol='Tumor_Sample_Barcode'):
    if 'Entrez' in genecol:
        mutdf = mutdf[mutdf[genecol].notnull()]
        mutdf[genecol] = mutdf[genecol].fillna(0).astype(int).astype(str)

    bindf = pd.DataFrame(0, index=np.unique(mutdf[genecol].values), columns=np.unique(mutdf[patcol].values))
    for genepat in mutdf[[genecol, patcol]].values:
        gene, pat = genepat[0], genepat[1]
        bindf.loc[gene, pat] = 1
    return bindf


def convert_cont_matrix2bin(df, limit=[-1.96, 1.96]):
    bindf = pd.DataFrame(0, index=df.index, columns=df.columns)
    bindf[df <= limit[0]]=1
    bindf[df >= limit[1]]=1
    return bindf


def combine_bins(lst, out_loc):
    out_loc = config.DATA_DIR / out_loc
    all_pats = np.unique(np.concatenate([l.columns for l in lst]))
    all_genes = np.unique(np.concatenate([l.index for l in lst]))
    df = pd.DataFrame(0, columns= all_pats, index=all_genes)
    df_deneme = df.copy()
    for l in lst:
        lg = l.groupby(l.index).sum()
        df_deneme[lg>0] = 1
    df_deneme.to_csv(out_loc, index=True)
    return df_deneme

def get_locs():
    loc_dict = coll.OrderedDict()
    for cancer in CANCERS:
        loc_dict[cancer]={}
        loc_dict[cancer]['mutation'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'{cancer.lower()}_tcga' / f'data_mutations_extended.txt'
        loc_dict[cancer]['std_expression'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'{cancer.lower()}_tcga' / f'data_RNA_Seq_v2_mRNA_median_all_sample_Zscores.txt'
        loc_dict[cancer]['expression'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'{cancer.lower()}_tcga' / f'{cancer}.rnaseqv2_norm.txt'
        loc_dict[cancer]['cnv'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'{cancer.lower()}_tcga' / f'data_CNA.txt'
        loc_dict[cancer]['sample_info'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'{cancer.lower()}_tcga' / f'data_bcr_clinical_data_sample.txt'
        loc_dict[cancer]['patient_info'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'{cancer.lower()}_tcga' / f'data_bcr_clinical_data_patient.txt'
        loc_dict[cancer]['gtex_expression'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'{cancer.lower()}_gtex' / f'{cancer.lower()}_RNASeQ_tpm.csv.gz'
    loc_dict['gtex_attribute'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'gtex' / 'GTEx_Analysis_v8_Annotations_SampleAttributesDS.txt'
    loc_dict['gtex_expression'] = config.DATA_DIR / 'raw_databases' / 'tcga_firehose' / f'gtex' / 'GTEx_Analysis_2017-06-05_v8_RNASeQCv1.1.9_gene_tpm.gct.gz'

    return loc_dict


def main():    
    
    colors = ['#7570b3', '#d95f02']
    colors2= ['#DDAA33', '#117733']
    group_labels =[f'WT', f'Mut']
    cancer_oncogene_marker_dict = {
                                   'Acute Myeloid Leukemia':('laml',{'FLT3': ['NRAS'], 'MGAM': ['TP53']}), 
                                   "B-Cell Non-Hodgkin's Lymphoma":('dlbc',{'REV3L': ['ARID1A'], 'VAV1': ['TP53'], 'ALK': ['TP53']}), 
                                   'Breast Carcinoma':('brca',{'BRD4': ['PIK3R1'], 'ROS1': ['CHD9'], 'MSI2': ['NR4A2']}),
                                   'Colorectal Carcinoma':('coadread',{'KRAS': ['PPP2R1A', 'ITSN1', 'KMT2D', 'NF1', 'AKAP9'], 'JAK3': ['MAP2K1'],
                                                                       'UBR5': ['FXR1'], 'MYBL1': ['MAP2K1'], 'KSR2': ['FXR1'], 'GNAS': ['FXR1'],
                                                                       'WWP1': ['FXR1', 'PIK3CA'], 'MECOM': ['FXR1'], 'NUP98': ['CNOT1', 'PIK3CA', 'PTEN'],
                                                                       'NTRK3': ['FXR1'], 'MTOR': ['FXR1']}), 
                                   'Esophageal Squamous Cell Carcinoma':('esca',{'WWTR1': ['EP300']}), 
                                   'Gastric Carcinoma':('stad',{'NSD2': ['ARID1A'], 'MECOM': ['EIF4G1']}), 
                                   'Glioblastoma':('gbm',{'IGF1R': ['EGFR']}), 
                                   'Melanoma':('skcm',{'MSI2': ['SVEP1'], 'PAK5': ['BPTF'], 'JAK3': ['ANK3']}), 
                                   'Non-Small Cell Lung Carcinoma':('luad',{'ADHFE1': ['STK11'], 'PDGFRB': ['MAP4K3'], 'NTRK3': ['KEAP1'], 
                                                                             'LGR5': ['CLSPN'], 'KDM5A': ['ARID1A'], 'NRAS': ['SETD2'],
                                                                             'MAP3K13': ['CLSPN'], 'MECOM': ['TP53BP1'], 'MGAM': ['TP53BP1']}), 
                                   'Ovarian Carcinoma':('ov',{'BCR': ['BRCA2', 'PIK3CA', 'SETD2'], 'ROS1': ['ATRX'],
                                                              'BRAF': ['PIK3CA', 'KMT2D'], 'LRP6': ['PIK3CA'], 'RPTOR': ['PIK3CA'], 'JAK3': ['PIK3CA'], 'VAV2': ['PIK3CA']}),  
                                   'Small Cell Lung Carcinoma':('luad',{'PAK5': ['BAZ2B', 'KMT2A'], 'CARD11': ['TJP1'], 'INSR': ['MED12']})
                                   }
    res_dict = {}
    for cancer, (cancer_tcga, oncogene_marker_dict) in cancer_oncogene_marker_dict.items():
        #cancer='Colorectal Carcinoma'
        mut = load_mut(cancer=cancer_tcga)
        mut_matrix = convert_mut2bin(mut, genecol='Hugo_Symbol', patcol='Tumor_Sample_Barcode')
        mut_dict = {gene: row[row==1].index.values for gene, row in mut_matrix.iterrows()}
        wt_dict = {gene: row[row==0].index.values for gene, row in mut_matrix.iterrows()}
        #clinical = load_sample_info(cancer='coadread')
        clinical = load_patient_info(cancer=cancer_tcga)
        expr_var_thold = 0.0001
        expr = load_expr(cancer=cancer_tcga, z_scored=False).fillna(0)
        z_scaled_expr = load_expr(cancer=cancer_tcga, z_scored=True).fillna(0)
        
        expr.index = expr.index.astype('str')
        z_scaled_expr.index = z_scaled_expr.index.astype('str')
        
        z_scaled_expr = z_scaled_expr.loc[expr.index!='nan',:]
        expr = expr.loc[expr.index!='nan',:]
        
        # Rename same index names
        duplicate_genes = expr.index[expr.index.duplicated(keep=False)].astype('str').values
        duplicate_rename_dict_counts = {}
        duplicate_rename_dict_z_scaled = {}
        unique_duplicates = np.unique(duplicate_genes, return_counts=True)
        for exp_gene, dup_count in zip(unique_duplicates[0], unique_duplicates[1]):
            duplicate_rename_dict_counts[exp_gene] = [f'{exp_gene}_{i}' for i in range(dup_count+1)]
            duplicate_rename_dict_z_scaled[exp_gene] = [f'{exp_gene}_{i}' for i in range(dup_count+1)]
        expr = expr.rename(index=lambda c: duplicate_rename_dict_counts[c].pop(0) if c in duplicate_rename_dict_counts.keys() else c)
        z_scaled_expr = z_scaled_expr.rename(index=lambda c: duplicate_rename_dict_z_scaled[c].pop(0) if c in duplicate_rename_dict_z_scaled.keys() else c)

        #expr = expr[expr.var(axis=1) >= expr_var_thold]
        if cancer_tcga not in res_dict.keys():
            res_dict[cancer_tcga] = {'CCLE_Cancer': cancer, 'mutation': {'mut': mut_dict, 'wt': wt_dict}, 'expression': expr, 'cases':{}}
        for oncogene, markers in oncogene_marker_dict.items():
            if oncogene in mut_dict.keys():
                onco_mut = mut_dict[oncogene].copy()
            else:
                print(f'{cancer}: {oncogene} cannot be found in mutation data')
                continue
            onco_wt = wt_dict[oncogene].copy()
            for marker in markers:
                print(f'{cancer}, {oncogene}, {marker} started!')
                marker_mut = mut_dict[marker].copy()
                marker_wt = wt_dict[marker].copy()
                onco_mut_marker_mut = np.intersect1d(onco_mut, marker_mut)
                onco_mut_marker_wt = np.intersect1d(onco_mut, marker_wt)
                onco_wt_marker_mut = np.intersect1d(onco_wt, marker_mut)
                onco_wt_marker_wt = np.intersect1d(onco_wt, marker_wt)
                onco_mut_marker_mut_n = len(onco_mut_marker_mut)
                onco_mut_marker_wt_n = len(onco_mut_marker_wt)
                
                expr_patients = expr.columns.values
                onco_mut_marker_mut = [asample for asample in onco_mut_marker_mut if asample in expr_patients]
                onco_mut_marker_wt = [asample for asample in onco_mut_marker_wt if asample in expr_patients]
                all_plot_pats = np.array([])
                all_plot_labels = np.array([])
                all_plot_colors = np.array([])
                all_plot_expr = np.array([])
                dict_per_dge_res = None
                if len(onco_mut_marker_mut)>0 and len(onco_mut_marker_wt)>0:
                    pdf_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_mut' / 'images' / 'mut_marker_dge_clustermap' / f'{cancer}_{oncogene}_{marker}_deseq2_noxticks.pdf'
                    res_per_dge_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_mut' / 'images' / 'mut_marker_dge_clustermap' / 'data' / f'{cancer}_{oncogene}_{marker}_deseq2_noxticks.pkl'
                    dict_per_dge_res = config.cache_numpy(res_per_dge_loc, is_pickle=True, is_full_loc=True)
                    if dict_per_dge_res is not None:
                        significant_dge_df = dict_per_dge_res['dge_df'] 
                        exp_scaled_counts_df = dict_per_dge_res['expression_zscored'] 
                        metadata_df = dict_per_dge_res['metadata'] 
                        if 'genes_tested' not in dict_per_dge_res.keys() or 'TCGA' in dict_per_dge_res['genes_tested'][0]:
                            patients=metadata_df.index.values
                            exp_counts_df = expr.loc[:,patients].astype(float).T
                            exp_counts_df[exp_counts_df < 0] = 0
                            exp_counts_df = exp_counts_df.loc[:, exp_counts_df.var(axis=0) >= expr_var_thold]
                            dict_per_dge_res['genes_tested'] = exp_counts_df.columns.values
                            config.cache_numpy(res_per_dge_loc, data=dict_per_dge_res, is_pickle=True, is_full_loc=True)
                    else:
                        dict_per_dge_res = {}
                        for agroup, grouplabel, color in zip([onco_mut_marker_wt, onco_mut_marker_mut], group_labels, colors):
                            all_plot_pats = np.concatenate((all_plot_pats, agroup), axis=None)
                            all_plot_labels = np.concatenate((all_plot_labels, [grouplabel]*len(agroup)), axis=None)
                            all_plot_colors = np.concatenate((all_plot_colors, [color]*len(agroup)), axis=None)
                            if len(all_plot_expr)==0:
                                all_plot_expr = expr.loc[:,agroup].values.T
                            else:
                                all_plot_expr = np.concatenate((all_plot_expr, expr.loc[:,agroup].values.T), axis=0)
                        if (oncogene, marker) not in res_dict[cancer_tcga]['cases'].keys():
                            res_dict[cancer_tcga]['cases'][(oncogene, marker)] = {'patients': all_plot_pats, 'labels': all_plot_labels, 'colors': all_plot_colors, 'expr': all_plot_expr}
                        exp_counts_df = pd.DataFrame(all_plot_expr, index=all_plot_pats, columns=expr.index.values).astype(float)
                        exp_counts_df[exp_counts_df < 0] = 0
                        exp_counts_df = exp_counts_df.loc[:, exp_counts_df.var(axis=0) >= expr_var_thold]
                        exp_rounded_count_df = exp_counts_df.round(0).astype(int)
                        exp_scaled_counts_df = z_scaled_expr.T.loc[all_plot_pats, exp_rounded_count_df.columns.values]
                        
                        metadata_df = pd.DataFrame(all_plot_labels, index=all_plot_pats, columns=[f'{marker} Status'])
                        '''
                            RSEM explanation of why we can use it for deseq2
                            https://bmcbioinformatics.biomedcentral.com/articles/10.1186/1471-2105-12-323
                            This count is generally a non-integer value and is the expectation of the number of alignable and unfiltered 
                            fragments that are derived from a isoform or gene given the ML abundances. These (possibly rounded) counts 
                            may be used by a differential expression method such as edgeR [9] or DESeq [8].
                        '''
                        inference = DefaultInference(n_cpus=8)
                        dds = DeseqDataSet(
                            counts=exp_rounded_count_df,
                            metadata=metadata_df,
                            design_factors=f'{marker} Status',  # compare samples based on the "condition"
                            # column ("B" vs "A")
                            refit_cooks=True,
                            inference=inference,
                        )
                        a = dds.deseq2()
                        stat_res = DeseqStats(dds, inference=inference, contrast=[f'{marker} Status', 'Mut', 'WT'])
                        stat_res.summary()
                        dge_res_df = stat_res.results_df
                        significant_dge_df = dge_res_df[dge_res_df['padj']<0.05]
                        significant_dge_df['LFC'] = (significant_dge_df['log2FoldChange']>0).astype('int').map({0:'<0', 1: '>0'})
                        print(significant_dge_df)
                        significant_dge_genes = significant_dge_df.index.values
                        count_df_with_dge = exp_counts_df[significant_dge_genes]
                        exp_scaled_counts_df = z_scaled_expr.loc[significant_dge_genes, all_plot_pats]
                        dict_per_dge_res['genes_tested'] = exp_counts_df.columns.values
                        dict_per_dge_res['dge_df'] = significant_dge_df
                        dict_per_dge_res['expression_zscored'] = exp_scaled_counts_df
                        dict_per_dge_res['metadata'] = metadata_df
                        config.cache_numpy(res_per_dge_loc, data=dict_per_dge_res, is_pickle=True, is_full_loc=True)
                    
                    if not pdf_loc.exists():
                        plt.clf()
                        col_lut = dict(zip(metadata_df[f'{marker} Status'].unique(), colors))
                        col_colors = metadata_df[f'{marker} Status'].map(col_lut)
                        
                        row_lut = dict(zip(significant_dge_df[f'LFC'].unique(), colors2))
                        row_colors = significant_dge_df[f'LFC'].map(row_lut)
                        
                        g = sns.clustermap(exp_scaled_counts_df, col_colors=col_colors, cmap="coolwarm", center=0, col_cluster=False, row_colors=row_colors)
                        ax = g.ax_heatmap
                        ax_cc = g.ax_col_colors
                        ax_rc = g.ax_row_colors
                        ax_bar = g.ax_cbar
                        title = f'{oncogene}-Mut, {cancer} Patients\n{marker}-Mut vs {marker}-WT DGE Analysis'
                        
                        leg_prop = {'size':14}
                        leg_title_prop = {'size':14, 'weight':'bold'}
                        #Add legends for column and row colors
                        col_handles = [Patch(facecolor=col_lut[name]) for name in col_lut]
                        legend1 = ax_cc.legend(col_handles, col_lut, title=f'{marker} Status', bbox_to_anchor=(0.35, 1.0), loc='lower center', alignment='left', prop=leg_prop, title_fontproperties=leg_title_prop)
                        #ax.add_artist(legend1)
                        
                        row_handles = [Patch(facecolor=row_lut[name]) for name in row_lut]
                        legend2 = ax_cc.legend(row_handles, row_lut, title=f'Log Fold Change', bbox_to_anchor=(0.65, 1.0), loc='lower center', alignment='left', prop=leg_prop, title_fontproperties=leg_title_prop)
                        ax_cc.add_artist(legend1)
                        
                        #Set figure title and x,y labels
                        g.fig.suptitle(title, x=0.6, y=0.98, fontsize=18, fontweight='bold', ha='center', va='top')
                        ax.set_xlabel(f'{oncogene} Mutated Patients', fontsize=16, fontweight='bold')
                        ax.set_ylabel('Diffentially Expressed Genes', fontsize=16, fontweight='bold')
                        ax_bar.set_ylabel('RSEM\nZ-scored', rotation=0, labelpad=12, fontweight='bold', fontsize=14, ha='left')
                        
                        # Adjust fonts of col_colors, row_colors and heatmap ticks
                        ax_cc.set_yticklabels(ax_cc.get_yticklabels(), fontsize=13, fontweight='bold')
                        ax_rc.set_xticklabels(ax_rc.get_xticklabels(), fontsize=13, fontweight='bold')
                        ax.set_xticks([])#ax.axes.xaxis.set_visible(False)
                        ax.set_xticklabels(ax.get_xticklabels(), fontsize=13)
                        ax.set_yticklabels(ax.get_yticklabels(), fontsize=13)
                        
                        config.ensure_dir(pdf_loc)
                        plt.savefig(pdf_loc, dpi=300, bbox_inches='tight')
                    '''
                    df_cluster = pd.DataFrame(all_plot_expr.T, columns=all_plot_pats, index=expr.index.values)
                    sns.clustermap(df_cluster.iloc[0:50,:], col_colors=all_plot_colors)
                    X = StandardScaler().fit_transform(all_plot_expr.copy())
                    for min_dist in [0.0, 0.1, 0.25]:
                        for n_neighbors in [2, 3]:
                            try:
                                umap_params = {'min_dist': min_dist, 'n_neighbors': n_neighbors}
                                umap_params_text = ''.join([f'{k[0]}{v}' for k, v in umap_params.items()])
                                umap_params_text = f'UMAP_{umap_params_text}'
                                pdf_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_mut' / 'images' / 'umap_pat_expr_with_mut_marker' / f'{cancer}_{oncogene}_{marker}_{umap_params_text}.pdf' #config.RESULTS_DIR / 'unsupervised_analysis' / 'entropy_categories' / f'{filepath.stem}_{feature_type}_{umap_params_text}_named.pdf'
                                plt.clf()
                                embedding = umap.UMAP(**umap_params).fit_transform(X.copy())
`                                fig, ax = plt.subplots()
                                for label, color in zip(group_labels, colors):
                                    mask = all_plot_labels == label
                                    ax.scatter(embedding[mask, 0], embedding[mask, 1], c=color, label=f'{label}')

                                title = cancer + ', ' + oncogene + ', ' + marker
                                plt.title(title)
                                plt.xlabel('UMAP 1')
                                plt.ylabel('UMAP 2')
                                sns.despine()
                                plt.legend()
                                config.ensure_dir(pdf_loc)
                                plt.savefig(pdf_loc, dpi=300, bbox_inches='tight')
                            except Exception as e:
                                print(f'{e}')
                    
                    onco_mut_marker_mut_mean = expr.loc[:,onco_mut_marker_mut].values.mean(1)
                    onco_mut_marker_wt_mean = expr.loc[:,onco_mut_marker_wt].values.mean(1)
                    onco_mut_marker_mut_var = expr.loc[:,onco_mut_marker_mut].values.var(1)
                    onco_mut_marker_wt_var = expr.loc[:,onco_mut_marker_wt].values.var(1)
                
                    z_score = (onco_mut_marker_mut_mean - onco_mut_marker_wt_mean) / np.sqrt(onco_mut_marker_mut_var/onco_mut_marker_mut_n + onco_mut_marker_wt_var/onco_mut_marker_wt_n)
                    upper2 = sum(z_score>2)
                    lower2 = sum(z_score<-2)
                    print(f'{oncogene} - {marker}: High={upper2} and Low={lower2}')
                    '''
                else:
                    print(f'Not enough data for {cancer} - {oncogene} - {marker}: {len(onco_mut_marker_mut)}, {len(onco_mut_marker_wt)}')
    print()
    #config.cache_numpy('data_for_dge/all', res_dict, is_pickle=True)
    '''
    loc_dict = get_locs()
    expr_df = load_gtex_expr(loc_dict['gtex_expression'])
    cancer_pat_dict = load_gtex_sample_info(loc_dict['gtex_attribute'])
    cancer2expr = {}
    for cancer, pats in cancer_pat_dict.items():
        col_list = np.union1d(['Name', 'Description'], pats)
        col_list = np.intersect1d(expr_df.columns.values, col_list)
        cancer2expr[cancer] = expr_df[col_list]
    '''

if __name__ == '__main__':
    main()
