import xenaPython as xena
from src.lib import config
import pandas as pd
import numpy as np

XENA_DATA_DIR = config.RAW_DATA_DIR / 'XENA'

def get_all_cohorts(hub=None, cohort_list=[], filters=[]):
    if hub is None:
        hub = "https://gdc.xenahubs.net"
    cohorts = xena.all_cohorts(hub, cohort_list)
    for afilter in filters:
        cohorts = [acohort for acohort in cohorts if afilter.lower() in acohort.lower()]
    for acohort in cohorts:
        dataset_list = xena.dataset_list(hub, [acohort])
        mutect_info = list(filter(lambda x: 'mutect2_snv' in x['name'], dataset_list))[0]
        samples = xena.dataset_samples (hub, mutect_info['name'], None)
        features = xena.dataset_field(hub, mutect_info['name'])
        data = xena.dataset_fetch(hub, mutect_info['name'], samples, features)
    print()
    
def get_mutation(cancer='TCGA-OV', detection='mutect2', hub='GDC_TCGA', filter_silent=True, filter_nonprimary=True):
    data_loc = XENA_DATA_DIR / f'/Volumes/iorio/Datasets/XENA/{hub}/{cancer}.{detection}_snv.tsv'
    mutation = pd.read_csv(data_loc, sep='\t', index_col=None)
    if filter_silent:
        mutation = mutation[~mutation['effect'].str.contains('silent')]
    if filter_nonprimary:
        mutation['sample_type'] = mutation['Sample_ID'].str.rsplit('-',expand=True).iloc[:,-1]
        mutation = mutation[mutation['sample_type'].str.contains('01')]
    return mutation
    
def convert_mut2bin(mutdf, genecol='gene', patcol=''):
    if 'Entrez' in genecol:
        mutdf = mutdf[mutdf[genecol].notnull()]
        mutdf[genecol] = mutdf[genecol].fillna(0).astype(int).astype(str)

    bindf = pd.DataFrame(0, index=np.unique(mutdf[genecol].values), columns=np.unique(mutdf[patcol].values))
    for genepat in mutdf[[genecol, patcol]].values:
        gene, pat = genepat[0], genepat[1]
        bindf.loc[gene, pat] = 1
    return bindf


def main():
    mut = get_mutation(cancer='TCGA-OV', detection='mutect2', hub='GDC_TCGA')
    mut_matrix = convert_mut2bin(mut, genecol='gene', patcol='Sample_ID')
    mut_dict = {gene: row[row==1].index.values for gene, row in mut_matrix.iterrows()}
    wt_dict = {gene: row[row==0].index.values for gene, row in mut_matrix.iterrows()}
    #get_all_cohorts(hub=None, cohort_list=[], filters=['tcga', 'ovarian'])


if __name__ == '__main__':
    main()