import sys, os
sys.path.insert(0, os.getcwd())
from src.lib import config
from src.process.results_process import previous_results
from src.process.data_process import extract_features

import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

def tested_gene_size(firstlevel_analysis='entropy'):
    search_folder = config.R_RESULTS_DIR
    list_of_dfs = []
    for filepath in search_folder.rglob(f'*_entropy_2022_*_{firstlevel_analysis}.tsv'):
        df = pd.read_csv(filepath, sep='\t', index_col=None)
        df['cancer'] = filepath.stem.split('_entropy_')[1].split('_', 1)[1].rsplit('_', 1)[0]
        df['variant'] = filepath.stem.split('_entropy_')[1].split('_', 1)[0]
        df['Test'] = df['cancer'] + ', ' + df['screenGene'] + ', ' + df['variant']
        list_of_dfs.append(df)
        
    all_dfs = pd.concat(list_of_dfs)
    all_dfs = all_dfs.sort_values(['screenGene', 'cancer', 'variant'])
    sns.histplot(all_dfs, y='Test', shrink=0.9)
    sns.despine()
    plt.xlabel('Number of Tested Candidate Genes')
    pdf_loc = config.RESULTS_DIR / 'previous_results' / 'histograms' / f'{firstlevel_analysis}_TestedCandidateGeneSizes.pdf'
    plt.savefig(pdf_loc, dpi=300, bbox_inches='tight')
    
    
def entropy_score_distribution(firstlevel_analysis='entropy'):
    search_folder = config.R_RESULTS_DIR
    for filepath in search_folder.rglob(f'*_entropy_2022_*_{firstlevel_analysis}.tsv'):
        plt.clf()
        df = pd.read_csv(filepath, sep='\t', index_col=None)
        cancer = filepath.stem.split('_entropy_')[1].split('_', 1)[1].rsplit('_', 1)[0]
        variant = filepath.stem.split('_entropy_')[1].split('_', 1)[0]
        main_gene = filepath.stem.split('_entropy_')[0]
        df = df[df['p']<0.05]
        title = cancer + ', ' + main_gene + ', ' + variant
        colors = ['red' if (x < 0.05) else 'gray' for x in df.p]
        sns.barplot(df, x='candidateGene', y='diffEntropy', palette = colors)
        plt.title(title)
        plt.xlabel('Candidate Gene')
        plt.ylabel('Differential Entropy')
        sns.despine()
        plt.xticks(rotation=90)
        pdf_loc = config.RESULTS_DIR / 'previous_results' / 'diff_entropy_distribution' / f'{filepath.stem}_only_significant.pdf'
        config.ensure_dir(pdf_loc)
        plt.savefig(pdf_loc, dpi=300, bbox_inches='tight')


def main():
    tested_gene_size()
    #entropy_score_distribution()
    
if __name__ == "__main__":
    main()
